/*
   Bordeaux : 1995
   */

#include "ext.h"

#include "adjust.h"


/* standard type of the binary functions that have as parameters two NFA and give
   a NFA as result */
typedef nfa (*nfa_bin_op)(nfa, nfa);
/* standard type of the unary functions that have as parameters one NFA and give
   a NFA as result */
typedef nfa (*nfa_una_op)(nfa);
/* standard type of the binary functions that have as parameters two DFA and give
   a DFA as result */
typedef dfa (*dfa_bin_op)(dfa, dfa);
/* standard type of the unary functions that have as parameters one DFA and give
   a DFA as result */
typedef dfa (*dfa_una_op)(dfa);


/* generic function to permit the use of DFA in place of NFA with binary 
   operations */
static dfa nfa_binary_operation_to_dfa(dfa X_dfa1, dfa X_dfa2, nfa_bin_op nfa_op);

/* generic function to permit the use of NFA in place of DFA with binary 
   operations */
static nfa dfa_binary_operation_to_nfa(nfa X_nfa1, nfa X_nfa2, dfa_bin_op dfa_op);

/* generic function to permit the use of DFA in place of NFA with the unary 
   operations */
static dfa nfa_unary_operation_to_dfa(dfa X_dfa, nfa_una_op nfa_op);

/* generic function to permit the use of NFA in place of DFA with the unary 
   operations */
static nfa dfa_unary_operation_to_nfa(nfa X_nfa, dfa_una_op dfa_op);

/* generic function to permit the use of NFA with inclusion and disjunction */
static boole generic_inclusion_nfa(nfa X_nfa1, nfa X_nfa2, boole inclusion);


/* ---------------------------- Global functions ------------------------------- */


/* ------------------------- Part 1 : tests ---------------------------- */

boole adjust_inclusion_dfa(dfa X_dfa1, dfa X_dfa2)
{
  return inclusion(X_dfa1, X_dfa2, TRUE);
}


boole adjust_disjunction_dfa(dfa X_dfa1, dfa X_dfa2)
{
  return inclusion(X_dfa1, X_dfa2, FALSE);
}


boole adjust_inclusion_nfa(nfa X_nfa1, nfa X_nfa2)
{
  return generic_inclusion_nfa(X_nfa1, X_nfa2, TRUE);
}

boole adjust_disjunction_nfa(nfa X_nfa1, nfa X_nfa2)
{
  return generic_inclusion_nfa(X_nfa1, X_nfa2, FALSE);
}


boole adjust_equiv_nfa(nfa X_nfa1, nfa X_nfa2)
{
  dfa   X_dfa1, X_dfa2;
  boole result;

  X_dfa1 = dfamdfa(nfa2dfa(X_nfa1), TRUE); /* the parameters of equiv must be */
  X_dfa2 = dfamdfa(nfa2dfa(X_nfa2), TRUE); /* minimal DFAs */

  result = equiv(X_dfa1, X_dfa2);
  
  fa_utils_free_temporary_fa(FALSE, 2, X_dfa1, X_dfa2);
  
  return result;
}

/* ------------ Part 2 : function manipulating only DFAs ---------------- */


dfa adjust_union_dfa(dfa X_dfa1, dfa X_dfa2)
{
  return nfa_binary_operation_to_dfa(X_dfa1, X_dfa2, unionfa);
}


dfa adjust_concat_dfa(dfa X_dfa1, dfa X_dfa2)
{
  return nfa_binary_operation_to_dfa(X_dfa1, X_dfa2, concatfa);
}


dfa adjust_shuffle_dfa(dfa X_dfa1, dfa X_dfa2)
{
  return nfa_binary_operation_to_dfa(X_dfa1, X_dfa2, shuffle);
}


dfa adjust_insec_dfa(dfa X_dfa1, dfa X_dfa2)
{
  return insecfa(X_dfa1, X_dfa2, FALSE);
}


dfa adjust_minus_dfa(dfa X_dfa1, dfa X_dfa2)
{
  return insecfa(X_dfa1, X_dfa2, TRUE);
}


dfa adjust_leftquot_dfa(dfa X_dfa1, dfa X_dfa2)
{
  nfa tmp_nfa;
  dfa result_dfa;

  tmp_nfa = leftquot(X_dfa1, X_dfa2);
  result_dfa = nfa2dfa(tmp_nfa);
  
  freenfa(tmp_nfa);

  return result_dfa;
}


dfa adjust_star_dfa(dfa X_dfa)
{
  return nfa_unary_operation_to_dfa(X_dfa, adjust_star_nfa);
}

dfa adjust_plus_dfa(dfa X_dfa)
{
  return nfa_unary_operation_to_dfa(X_dfa, adjust_plus_nfa);
}


dfa adjust_rev_dfa(dfa X_dfa)
{
  return nfa_unary_operation_to_dfa(X_dfa, revnfa);
}


dfa adjust_suff_dfa(dfa X_dfa)
{
  nfa result_nfa;
  dfa result_dfa;

  result_nfa = suff(X_dfa);
  result_dfa = nfa2dfa(result_nfa);

  freenfa(result_nfa);
  
  return result_dfa;
}


/* ------------ Part 2 : function manipulating only NFAs ---------------- */

nfa adjust_insec_nfa(nfa X_nfa1, nfa X_nfa2)
{
  return dfa_binary_operation_to_nfa(X_nfa1, X_nfa2, adjust_insec_dfa);
}


nfa adjust_minus_nfa(nfa X_nfa1, nfa X_nfa2)
{
  return dfa_binary_operation_to_nfa(X_nfa1, X_nfa2, adjust_minus_dfa);
}


nfa adjust_leftquot_nfa(nfa X_nfa1, nfa X_nfa2)
{
  dfa X_dfa1, X_dfa2;
  nfa result_nfa;

  X_dfa1 = nfa2dfa(X_nfa1);
  X_dfa2 = nfa2dfa(X_nfa2);

  result_nfa = leftquot(X_dfa1, X_dfa2);
  
  fa_utils_free_temporary_fa(FALSE, 2, X_dfa1, X_dfa2);

  return result_nfa;
}


nfa adjust_rightquot_nfa(nfa X_nfa1, nfa X_nfa2)
{
  return dfa_binary_operation_to_nfa(X_nfa1, X_nfa2, rightquot);
}


nfa adjust_star_nfa(nfa X_nfa)
{
  return starnfa(X_nfa, FALSE);
}

nfa adjust_plus_nfa(nfa X_nfa)
{
  return starnfa(X_nfa, TRUE);
}


nfa adjust_compl_nfa(nfa X_nfa)
{
  return dfa_unary_operation_to_nfa(X_nfa, compldfa);
}


nfa adjust_minL_nfa(nfa X_nfa)
{
  return dfa_unary_operation_to_nfa(X_nfa, minL);
}


nfa adjust_maxL_nfa(nfa X_nfa)
{
  return dfa_unary_operation_to_nfa(X_nfa, maxL);
}


nfa adjust_pref_nfa(nfa X_nfa)
{
  return dfa_unary_operation_to_nfa(X_nfa, pref);
}


nfa adjust_suff_nfa(nfa X_nfa)
{
  dfa X_dfa;
  nfa result_nfa;

  X_dfa = nfa2dfa(X_nfa);
  result_nfa = suff(X_dfa);

  freedfa(X_dfa);
  
  return result_nfa;
}


/* ---------------------------- Static functions ------------------------------- */


static boole generic_inclusion_nfa(nfa X_nfa1, nfa X_nfa2, boole binclusion)
{
  dfa   tmp_dfa1, tmp_dfa2;
  boole result;

  tmp_dfa1 = nfa2dfa(X_nfa1);
  tmp_dfa2 = nfa2dfa(X_nfa2);

  result = inclusion(tmp_dfa1, tmp_dfa2, binclusion);

  fa_utils_free_temporary_fa(FALSE, 2, tmp_dfa1, tmp_dfa2);

  return result;
}



static dfa nfa_binary_operation_to_dfa(dfa X_dfa1, dfa X_dfa2, nfa_bin_op nfa_op)
{
  nfa X_nfa1;
  nfa X_nfa2;
  nfa result_nfa;
  dfa result_dfa;

  /* conversion of the parameters into NFAs */
  X_nfa1 = dfa2nfa(X_dfa1);  
  X_nfa2 = dfa2nfa(X_dfa2);

  /* computation */
  result_nfa = nfa_op(X_nfa1, X_nfa2);
  /* conversion of the result into a DFA */
  result_dfa = nfa2dfa(result_nfa);

  fa_utils_free_temporary_fa(TRUE, 3, X_nfa1, X_nfa2, result_nfa);

  return result_dfa;
}


static nfa dfa_binary_operation_to_nfa(nfa X_nfa1, nfa X_nfa2, dfa_bin_op dfa_op)
{
  dfa X_dfa1;
  dfa X_dfa2;
  dfa result_dfa;
  nfa result_nfa;

  /* conversion of the parameters into NFAs */
  X_dfa1 = nfa2dfa(X_nfa1);  
  X_dfa2 = nfa2dfa(X_nfa2);

  /* computation */
  result_dfa = dfa_op(X_dfa1, X_dfa2);
  /* conversion of the result into a DFA */
  result_nfa = dfa2nfa(result_dfa);

  fa_utils_free_temporary_fa(FALSE, 3, X_dfa1, X_dfa2, result_dfa);

  return result_nfa;
}


static dfa nfa_unary_operation_to_dfa(dfa X_dfa, nfa_una_op nfa_op)
{
  nfa X_nfa;
  nfa result_nfa;
  dfa result_dfa;

  /* conversion of the parameter into a NFA */
  X_nfa = dfa2nfa(X_dfa);

  /* computation */
  result_nfa = nfa_op(X_nfa);
  /* conversion of the result into a DFA */
  result_dfa = nfa2dfa(result_nfa);

  fa_utils_free_temporary_fa(TRUE, 2, X_nfa, result_nfa);

  return result_dfa;
}



static nfa dfa_unary_operation_to_nfa(nfa X_nfa, dfa_una_op dfa_op)
{
  dfa X_dfa;
  dfa result_dfa;
  nfa result_nfa;

  /* conversion of the parameter into a NFA */
  X_dfa = nfa2dfa(X_nfa);

  /* computation */
  result_dfa = dfa_op(X_dfa);
  /* conversion of the result into a DFA */
  result_nfa = dfa2nfa(result_dfa);

  fa_utils_free_temporary_fa(FALSE, 2, X_dfa, result_dfa);

  return result_nfa;
}











