#ifndef CODETEST_H
#define CODETEST_H

/*
   Bordeaux : 1995
   */


#include "ext.h"

/* This module implements a set of tests on the notion of code and stack-code :
    - for the notion of code, see :
      [1] "Theory of codes", J.Berstel, D.Perrin, Academic Press, Orlando, 1985
      [2] "Outfix and infix codes and related classes of languages", Ito, Jurgensen,
          Shyr, Thierrin, Journal of Computer and System Sciences, 1991, p 484-508
    - for the notion of stack-code see :
      [3] "On stack languages", B.Patrou I.Litovsky B.Le Saec, Algebraic Theory and 
          Applications to Formal Languages and Codes, 1993, p 174-182 */


/* In all the procedure of this module, assume that the parameters (DFA and NFA) are 
   correct 
   Moreover, it is recommended, when not explicitely required, that the automatons
   are minimal */


/* test if a language (of DFA X_dfa) is a code : see [1] p 51 
   assume that X_dfa is minimal */
boole codetest_code(dfa X_dfa);

/* test if a language (of DFA X_dfa and NFA X_nfa) is a stack-code : see [3] 
   assume that X_dfa is a minimal DFA */
boole codetest_stack_code(dfa X_dfa, nfa X_nfa);


/* In the following tests, we assume that the tested language is a code */

/* test if a code (of DFA X_dfa) is a prefix code : see [2] */
boole codetest_prefix_code(dfa X_dfa);

/* test if a code (of DFA X_dfa) is a suffix code : see [2] */
boole codetest_suffix_code(dfa X_dfa);

/* test if a code X (of DFA X_dfa) is a bifix code : see [2] 
   if X is a prefix (suffix) code : prefix (suffix) = TRUE
   if X is not a prefix (suffix) code : prefix (suffix) = FALSE
   else prefix (suffix) = UN_KNOWN */
boole codetest_bifix_code(dfa X_dfa, boolx prefix, boolx suffix);

/* test if a code (of DFA X_dfa and NFA X_nfa) is a p_infix code : see [2] 
   assume that X_nfa is minimal */
boole codetest_p_infix_code(nfa X_nfa, dfa X_dfa);

/* test if a code (of DFA X_dfa and NFA X_nfa) is a s_infix code : see [2] 
   assume that X_nfa is minimal */
boole codetest_s_infix_code(nfa X_nfa, dfa X_dfa);

/* test if a code X (of DFA X_dfa and NFA X_nfa) is an infix code : see [2] 
   if X is a p_infix (s_infix) code : p_infix (s_infix) = TRUE
   if X is not a p_infix (s_infix) code : p_infix (s_infix) = FALSE
   else p_infix (s_infix) = UN_KNOWN 
   assume that X_nfa is minimal */
boole codetest_infix_code(nfa X_nfa, dfa X_dfa, boolx p_infix, boolx s_infix);

/* test if a code (of DFA X_dfa) is an outfix code : see [2] 
   assume that X_dfa is minimal */
boole codetest_outfix_code(dfa X_dfa);

/* test if a code (of DFA X_dfa and NFA X_nfa) is a hypercode : see [2] 
   assume that X_dfa is minimal */
boole codetest_hypercode(dfa X_dfa, nfa X_nfa);

/* test if a code (of DFA X_dfa) is an uniform code : see [2] 
   assume that X_dfa is minimal */
boole codetest_uniform_code(dfa X_dfa);

/* test if a code X (of DFA X_dfa and NFA X_nfa) on an alphabet A is a full uniform 
   code (see [2]) :
   full uniform code <==> strict right semaphore 
                          (X is prefix and (A+).X in X.(A+))
   full uniform code <==> strict leftt semaphore 
                          (X is suffix and X.(A+) in (A+).X) 
   assume that X_dfa is minimal */
boole codetest_full_uniform_code(dfa X_dfa);

/* test if a code (of NFA X_nfa and DFA X_dfa) is a right semaphore code : see [2]
   if X is a prefix code : prefix = TRUE
   if X is not a prefix code : prefix = FALSE
   else prefix = UN_KNOWN 
   assume that X_nfa is minimal */
boole codetest_right_semaphore_code(nfa X_nfa, dfa X_dfa, boolx prefix);


/* test if a code (of NFA X_nfa and DFA X_dfa) is a left semaphore code : see [2] 
   if X is a suffix code : suffix = TRUE
   if X is not a suffix code : suffix = FALSE
   else suffix = UN_KNOWN 
   assume that X_nfa is minimal */
boole codetest_left_semaphore_code(nfa X_nfa, dfa X_dfa, boolx suffix);


#endif

